#include "CInterface.h"
#include "CWorld.h"

#include <SDL/SDL.h>

#include <persist.h>

#include <qapplication.h>
#include <qlabel.h>
#include <qstring.h>
 
#ifdef __APPLE__
	#include <GLUT/glut.h>
#else
	#include <GL/glut.h>
#endif

#ifdef WIN32
#  include <time.h>
#include <direct.h>
#endif

//-- set the mobinet version
#define VERSION "MobiNet v1.1.100"
unsigned char VER_MAJ = 1 ;		//major version
unsigned char VER_MIN = 1 ;		//minor version
unsigned char VER_REV = 100 ;	//revision
 
void help()
{
  cerr << "-=-=-=-=-=-=- MobiNet -=-=-=-=-=-=-=" << endl;
  cerr << "                                    " << endl;
  cerr << " Command line options:              " << endl;
  cerr << "   -client <host> <port> <sessionFile>" << endl;
  cerr << "   -nosave    : disable saving buttons" << endl;
  cerr << "   -nopresets : disable preset menus" << endl;
  cerr << "   -nohotkey  : disable hotkeys     " << endl;
  cerr << "   -mobilesdir <directory> : set the default mobiles directory" << endl;
  cerr << "   -presetsdir <directory> : set the presets directory" << endl;
  cerr << "                                    " << endl;
  cerr << "                                    " << endl;
  cerr << "                   " << VERSION << "   " << endl;
  cerr << "-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=" << endl;
  exit (0); 
}

int main(int argc, char** argv)
{
  bool  save    = true;
  bool  presets = true;
  bool  hotkeys = true;
  bool  client=false;
  int   client_port=6333;
  char *client_host=NULL;
#ifdef LINUX_PACKAGE_INSTALL
  char * mobilesDir("/usr/share/mobinet/mobiles");
  char * presetsDir("/usr/share/mobinet/presets");
#else
	#ifdef __APPLE__
		//char * mobilesDir("../../../../../mobiles");
		//char * presetsDir("../../../../../presets");
		char * mobilesDir("../mobiles");
		char * presetsDir("../presets");
	#else
		char * mobilesDir("../../mobiles");
		char * presetsDir("../../presets");
	#endif
#endif

  cerr <<endl<< "***--- " << VERSION << " ---*** " << endl <<endl;

  QString sessionFile;

  SDL_Init(0);
  glutInit(&argc,argv);

  srand(time(NULL));  
#ifndef WIN32
  srand48(time(NULL));
#endif
  // set application version
  PERSIST_APPLICATION_VERSION(VERSION);
  // Read command lines arguments.
  QApplication application(argc,argv);

//go to the mobinet directory
  std::string dir(argv[0]);
  std::string::size_type pos = dir.find_last_of("/\\");
  if (pos != std::string::npos){
#ifndef WIN32
	  chdir(dir.substr(0,pos).c_str());
#else
      _chdir(dir.substr(0,pos).c_str());
#endif
  }


  for (int i=1;i<argc;)
  {
    if (!strcmp(argv[i],"-client"))
    {
      if (argc-i == 3)
      {
        client=true;
        client_host=argv[i+1];
        client_port=atoi(argv[i+2]);
        i+=3;
      }
      else if (argc-i == 2)
      {
        client=true;
        client_host=argv[i+1];
        i+=2;
      }
    } 
    else if (strcmp(argv[i],"-nosave")==0)
    {
      save = false;
      i++;
    }
    else if (strcmp(argv[i],"-nopresets")==0)
    {
      presets = false;
      i++;
    }
    else if (strcmp(argv[i],"-nohotkey")==0)
    {
      hotkeys = false;
      i++;
    }
    else if (strcmp(argv[i],"-h")==0 || strcmp(argv[i],"-help")==0 || strcmp(argv[i],"--help")==0)
    {
      help();
      i++;
    }
	else if (strcmp(argv[i],"-mobilesdir")==0)
    {
      mobilesDir = argv[i+1];
      i+=2;
    }
	else if (strcmp(argv[i],"-presetsdir")==0)
    {
      presetsDir = argv[i+1];
      i+=2;
    }
    else {
		std::string s(argv[i]);
		int ind= s.find_last_of('.');
		if (ind != string::npos && s.substr(ind, s.size()-1) == ".session"){
			sessionFile=argv[i];
			i++;
		}else{
			cerr << "ERREUR: je ne connais pas l'option " << argv[i] << endl<<endl;
			help();
		}
      // i++;
    }
  }

  // Instantiate the interface.
  g_MainWindow = new CInterface(NULL,0,0, presetsDir, mobilesDir);
  if (!save)    g_MainWindow->activeSave(false);
  if (!presets) g_MainWindow->activePresets(false);
  if (!hotkeys) g_MainWindow->activeHotkeys(false);
  
  // Make the viewer window visible on screen.
  g_MainWindow->setCaption(VERSION);
  g_MainWindow->showMaximized();
  if(!sessionFile.isEmpty())  g_MainWindow->loadSession(sessionFile);

  // Start network
  try
  {
  if (client)
  {
    NETWORK.setClientUDPPort(client_port);
    NETWORK.startClient(client_host, VER_MAJ, VER_MIN, VER_REV);
  }
  else
    NETWORK.startServer(VER_MAJ, VER_MIN, VER_REV);
  }
  catch (...)
  {
    cerr << "Impossible de lancer le reseau." << endl;
  }

  // network ?
  if (NETWORK.isEnabled())
  {
    WORLD.waitForNetworkReady();
  }

  if (client)
    g_MainWindow->m_LblClientNumber->setText(QString("%1").arg(NETWORK.clientId()+1));
  else
  {
    g_MainWindow->m_LblClientNumber->setText(QString("%1 (server)").arg(NETWORK.clientId()+1));
  }
  // Set the viewer as the application main widget.
  application.setMainWidget(g_MainWindow);

  PERSIST.setMerge(true);
  PERSIST_AFTER_LOAD;

  try
  {
    // Run main loop.
    return application.exec();
  }
  catch (...)
  {
    cerr << "Fatal error during execution." << endl;
  }

  return (0);
}
